@@ -374,6 +374,15 @@ typedef struct gs_param_list_procs_s {
 #define param_commit(plist)\
          (*(plist)->procs->commit)(plist)
 
+         /*
+          * Read the value of a previously signalled error.  (Only used when reading.)
+          */
+#define param_proc_read_signalled_error(proc)\
+         int proc(gs_param_list *, gs_param_name)
+         param_proc_read_signalled_error((*read_signalled_error));
+#define param_read_signalled_error(plist, pkey)\
+         (*(plist)->procs->read_signalled_error)(plist, pkey)
+
 } gs_param_list_procs;
 
 /* Transmit typed parameters. */
@@ -477,7 +486,8 @@ typedef struct gs_param_item_s {
  * the optional default_obj, the item isn't transferred.
  */
 int gs_param_read_items(gs_param_list * plist, void *obj,
-                        const gs_param_item_t * items);
+                        const gs_param_item_t * items,
+                        gs_memory_t *mem);
 int gs_param_write_items(gs_param_list * plist, const void *obj,
                          const void *default_obj,
                          const gs_param_item_t * items);
