@@ -22,7 +22,7 @@ word/document.xml. */
 
 
 int extract_docx_write_template(
-        extract_alloc_t*    alloc, 
+        extract_alloc_t*    alloc,
         extract_astring_t*  contentss,
         int                 contentss_num,
         images_t*           images,
@@ -34,7 +34,7 @@ int extract_docx_write_template(
 
 Uses the 'zip' and 'unzip' commands internally.
 
-contents
+contentss
 contentss_num
     Content to be inserted into word/document.xml.
 document
